@@ -376,27 +376,33 @@ static int handle_configure_request(xcb_configure_request_event_t *event) {
             bsr.height -= deco_height;
         }
         Con *floatingcon = con->parent;
-        DLOG("Container is a floating leaf node, will do that.\n");
+
+        Rect newrect = floatingcon->rect;
+
         if (event->value_mask & XCB_CONFIG_WINDOW_X) {
-            floatingcon->rect.x = event->x + (-1) * bsr.x;
-            DLOG("proposed x = %d, new x is %d\n", event->x, floatingcon->rect.x);
+            newrect.x = event->x + (-1) * bsr.x;
+            DLOG("proposed x = %d, new x is %d\n", event->x, newrect.x);
         }
         if (event->value_mask & XCB_CONFIG_WINDOW_Y) {
-            floatingcon->rect.y = event->y + (-1) * bsr.y;
-            DLOG("proposed y = %d, new y is %d\n", event->y, floatingcon->rect.y);
+            newrect.y = event->y + (-1) * bsr.y;
+            DLOG("proposed y = %d, new y is %d\n", event->y, newrect.y);
         }
         if (event->value_mask & XCB_CONFIG_WINDOW_WIDTH) {
-            floatingcon->rect.width = event->width + (-1) * bsr.width;
-            floatingcon->rect.width += con->border_width * 2;
-            DLOG("proposed width = %d, new width is %d (x11 border %d)\n", event->width, floatingcon->rect.width, con->border_width);
+            newrect.width = event->width + (-1) * bsr.width;
+            newrect.width += con->border_width * 2;
+            DLOG("proposed width = %d, new width is %d (x11 border %d)\n",
+                 event->width, newrect.width, con->border_width);
         }
         if (event->value_mask & XCB_CONFIG_WINDOW_HEIGHT) {
-            floatingcon->rect.height = event->height + (-1) * bsr.height;
-            floatingcon->rect.height += con->border_width * 2;
-            DLOG("proposed height = %d, new height is %d (x11 border %d)\n", event->height, floatingcon->rect.height, con->border_width);
+            newrect.height = event->height + (-1) * bsr.height;
+            newrect.height += con->border_width * 2;
+            DLOG("proposed height = %d, new height is %d (x11 border %d)\n",
+                 event->height, newrect.height, con->border_width);
         }
-        floating_maybe_reassign_ws(floatingcon);
-        tree_render();
+
+        DLOG("Container is a floating leaf node, will do that.\n");
+        floating_reposition(floatingcon, newrect);
+        return 1;
     }
 
     /* Dock windows can be reconfigured in their height */
@@ -452,10 +458,20 @@ static int handle_screen_change(xcb_generic_event_t *e) {
  *
  */
 static int handle_unmap_notify_event(xcb_unmap_notify_event_t *event) {
-    // XXX: this is commented out because in src/x.c we disable EnterNotify events
-    /* we need to ignore EnterNotify events which will be generated because a
-     * different window is visible now */
-    //add_ignore_event(event->sequence, XCB_ENTER_NOTIFY);
+    /* If the client (as opposed to i3) destroyed or unmapped a window, an
+     * EnterNotify event will follow (indistinguishable from an EnterNotify
+     * event caused by moving your mouse), causing i3 to set focus to whichever
+     * window is now visible.
+     *
+     * In a complex stacked or tabbed layout (take two v-split containers in a
+     * tabbed container), when the bottom window in tab2 is closed, the bottom
+     * window of tab1 is visible instead. X11 will thus send an EnterNotify
+     * event for the bottom window of tab1, while the focus should be set to
+     * the remaining window of tab2.
+     *
+     * Therefore, we ignore all EnterNotify events which have the same sequence
+     * as an UnmapNotify event. */
+    add_ignore_event(event->sequence, XCB_ENTER_NOTIFY);
 
     DLOG("UnmapNotify for 0x%08x (received from 0x%08x), serial %d\n", event->window, event->event, event->sequence);
     Con *con = con_by_window_id(event->window);
@@ -968,6 +984,12 @@ static int handle_focus_in(xcb_focus_in_event_t *event) {
         return 1;
     }
 
+    /* Skip dock clients, they cannot get the i3 focus. */
+    if (con->parent->type == CT_DOCKAREA) {
+        DLOG("This is a dock client, not focusing.\n");
+        return 1;
+    }
+
     DLOG("focus is different, updating decorations\n");
 
     /* Get the currently focused workspace to check if the focus change also
